#include "config.h"

#include "gettext.h"
#include "trayicon.h"
#include "quarkclient.h"
#include "editor.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gconf/gconf-client.h>
#include <string.h>    /* for strcmp() */
#include <unistd.h>    /* for sleep() */

#define QUARK_ICON "quark-icon"

#define QUARK_GCONF_ROOT "/apps/quark"
#define LOOP_PLAYLIST    QUARK_GCONF_ROOT "/loop_playlist"
#define RANDOM_ORDER     QUARK_GCONF_ROOT "/random_order"
#define PLAY_ON_RUN      QUARK_GCONF_ROOT "/play_at_startup"

static GtkWidget *trayicon;
static GtkWidget *menu;
static GtkWidget *editor;
static GtkCheckMenuItem *loop_playlist_item;
static GtkCheckMenuItem *random_order_item;
static GtkCheckMenuItem *play_on_run_item;

static QuarkClient *qc;
static GConfClient *gconf;

static GdkPixbuf *applet_icon = NULL;

static gboolean
menu_quit(GtkWidget *item, gpointer data)
{
    gtk_main_quit();
    return FALSE;
}

static void
applet_error (QuarkClient *qc, GError *error, gpointer data)
{
    GtkWidget *dia;

    dia = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR,
                                  GTK_BUTTONS_CLOSE,
                                  "%s", error->message);
    gtk_dialog_run (GTK_DIALOG (dia));
    gtk_widget_destroy (dia);
    g_error_free (error);
}

static void
applet_play (GtkMenuItem *item, gpointer data)
{
    QuarkClient *qc = QUARKCLIENT (data);
    quarkclient_play (qc, NULL);
}

static void
applet_pause (GtkMenuItem *item, gpointer data)
{
    QuarkClient *qc = QUARKCLIENT (data);
    quarkclient_pause (qc, NULL);
}

static void
applet_stop (GtkMenuItem *item, gpointer data)
{
    QuarkClient *qc = QUARKCLIENT (data);
    quarkclient_stop (qc, NULL);
}

static void
applet_next_track (GtkMenuItem *item, gpointer data)
{
    QuarkClient *qc = QUARKCLIENT (data);
    quarkclient_next_track (qc, NULL);
}

static void
applet_previous_track (GtkMenuItem *item, gpointer data)
{
    QuarkClient *qc = QUARKCLIENT (data);
    quarkclient_previous_track (qc, NULL);
}

static void
applet_clear (GtkMenuItem *item, gpointer data)
{
    editor_clear_files (EDITOR (editor));
}

static void
applet_append_files (GtkMenuItem *item, gpointer data)
{
    editor_add_files (EDITOR (editor), TRUE);
}

static void
applet_replace_files (GtkMenuItem *item, gpointer data)
{
    editor_add_files (EDITOR (editor), FALSE);
}

static void
applet_edit_playlist (GtkMenuItem *item, gpointer data)
{
    gtk_widget_show (editor);
}

static void
applet_help (GtkMenuItem *item, gpointer data)
{
}

static void
applet_about (GtkMenuItem *item, gpointer data)
{
}

static void
applet_loop (GtkCheckMenuItem *item, gpointer data)
{
    gconf_client_set_bool (gconf, LOOP_PLAYLIST,
                           gtk_check_menu_item_get_active (item), NULL);
}

static void
applet_random (GtkCheckMenuItem *item, gpointer data)
{
    gconf_client_set_bool (gconf, RANDOM_ORDER,
                           gtk_check_menu_item_get_active (item), NULL);
}

static void
applet_play_on_run (GtkCheckMenuItem *item, gpointer data)
{
    gconf_client_set_bool (gconf, PLAY_ON_RUN,
                           gtk_check_menu_item_get_active (item), NULL);
}

static gboolean
applet_user_event (GtkWidget *widget, GdkEvent *event, gpointer data)
{
    QuarkClient *qc = QUARKCLIENT (data);

    g_message ("event");

    switch (event->type) {
    case GDK_BUTTON_PRESS:
        switch (event->button.button) {
        case 1: /* left */
            quarkclient_pause (qc, NULL);
            break;
        case 2: /* middle */
            editor_add_files (EDITOR (editor),
                              event->button.state & GDK_SHIFT_MASK);
            break;
        case 3: /* right */
            /* note: if you're gunna bind this, then you should return TRUE
               so the menu doesn't pop up probably.. */
            break;
        }
        break;
    case GDK_SCROLL:
        switch (event->scroll.direction) {
        case GDK_SCROLL_LEFT:
        case GDK_SCROLL_UP:
            quarkclient_previous_track (qc, NULL);
            break;
        case GDK_SCROLL_RIGHT:
        case GDK_SCROLL_DOWN:
            quarkclient_next_track (qc, NULL);
            break;
        }
        break;
    default:
        break;
    }
    return FALSE;
}

static void
applet_load_stock (QuarkClient *qc)
{
    GtkIconFactory *factory;
    GError *e = NULL;

    gtk_icon_factory_add_default (factory = gtk_icon_factory_new ());

    applet_icon = gdk_pixbuf_new_from_file (PIXMAPDIR G_DIR_SEPARATOR_S
                                            "quark.png", &e);
    if (!applet_icon) {
        gchar *msg = g_strdup_printf 
            (_("Failed to load the Quark icon, quark is probably not "
               "installed correctly. The error given was '%s'."),
             e->message);
        g_free (e->message);
        e->message = msg;
        applet_error (qc, e, NULL);
    } else {
        GtkIconSet *set;

        set = gtk_icon_set_new_from_pixbuf (applet_icon);
        gtk_icon_factory_add (factory, QUARK_ICON, set);
        gtk_icon_set_unref (set);
    }
}

static void
applet_gconf_changed (GConfClient *gconf,
                      guint cnxn_id,
                      GConfEntry *entry,
                      gpointer data)
{
    g_message ("notify %s", entry->key);
    if (!strcmp (entry->key, LOOP_PLAYLIST)) {
        gtk_check_menu_item_set_active (loop_playlist_item,
                                        gconf_value_get_bool (entry->value));
    } else if (!strcmp (entry->key, RANDOM_ORDER)) {
        gtk_check_menu_item_set_active (random_order_item,
                                        gconf_value_get_bool (entry->value));
    } else if (!strcmp (entry->key, PLAY_ON_RUN)) {
        gtk_check_menu_item_set_active (play_on_run_item,
                                        gconf_value_get_bool (entry->value));
    }
}

static void
applet_song_change (QuarkClient *qc, QuarkClientTrack *curtrack, gpointer data)
{
    TrayIcon *ti = TRAYICON (data);
    gchar *name = g_path_get_basename (quarkclient_track_path (curtrack));
    trayicon_set_tooltip (ti, name);
    g_free (name);   
}

void
applet_startup ()
{
    GtkWidget *item, *image;

    qc = quarkclient_new ();
    quarkclient_set_default_error_handler (qc, applet_error, trayicon);
    quarkclient_open (qc, "strange", NULL);

    gconf = gconf_client_get_default ();
    gconf_client_add_dir (gconf, QUARK_GCONF_ROOT,
                          GCONF_CLIENT_PRELOAD_NONE, NULL);
    gconf_client_notify_add (gconf, QUARK_GCONF_ROOT, applet_gconf_changed,
                             NULL, NULL, NULL);

    applet_load_stock (qc);

    editor = editor_new(qc, gconf, applet_icon);

    menu = gtk_menu_new();

    item = gtk_image_menu_item_new_with_mnemonic (_("_Play"));
    image = gtk_image_new_from_stock (GTK_STOCK_YES, GTK_ICON_SIZE_MENU);
    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "activate", G_CALLBACK(applet_play), qc);

    item = gtk_image_menu_item_new_with_mnemonic (_("Pa_use"));
    image = gtk_image_new_from_stock (GTK_STOCK_NO, GTK_ICON_SIZE_MENU);
    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "activate", G_CALLBACK(applet_pause), qc);

    item = gtk_image_menu_item_new_with_mnemonic (_("_Stop"));
    image = gtk_image_new_from_stock (GTK_STOCK_STOP, GTK_ICON_SIZE_MENU);
    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "activate", G_CALLBACK(applet_stop), qc);

    gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                           gtk_separator_menu_item_new());

    item = gtk_image_menu_item_new_with_mnemonic (_("_Next track"));
    image = gtk_image_new_from_stock (GTK_STOCK_GO_FORWARD,
                                      GTK_ICON_SIZE_MENU);
    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "activate", G_CALLBACK(applet_next_track), qc);

    item = gtk_image_menu_item_new_with_mnemonic (_("Pre_vious track"));
    image = gtk_image_new_from_stock (GTK_STOCK_GO_BACK, GTK_ICON_SIZE_MENU);
    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "activate", G_CALLBACK(applet_previous_track), qc);

    gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                           gtk_separator_menu_item_new());

    item = gtk_image_menu_item_new_with_mnemonic (_("_Load songs..."));
    image = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "activate", G_CALLBACK(applet_replace_files), qc);

    item = gtk_image_menu_item_new_with_mnemonic (_("_Append songs..."));
    image = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU);
    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "activate", G_CALLBACK(applet_append_files), qc);

    item = gtk_image_menu_item_new_with_mnemonic (_("_Clear playlist"));
    image = gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU);
    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "activate", G_CALLBACK(applet_clear), qc);

    gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                           gtk_separator_menu_item_new());

    item = gtk_check_menu_item_new_with_mnemonic (_("Play on startup"));
    play_on_run_item = GTK_CHECK_MENU_ITEM (item);
    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
                                    gconf_client_get_bool (gconf,
                                                           PLAY_ON_RUN,
                                                           NULL));
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "toggled", G_CALLBACK(applet_play_on_run), qc);

    item = gtk_check_menu_item_new_with_mnemonic (_("Loop playlist"));
    loop_playlist_item = GTK_CHECK_MENU_ITEM (item);
    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
                                    gconf_client_get_bool (gconf,
                                                           LOOP_PLAYLIST,
                                                           NULL));
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "toggled", G_CALLBACK(applet_loop), qc);

    item = gtk_check_menu_item_new_with_mnemonic (_("Random order"));
    random_order_item = GTK_CHECK_MENU_ITEM (item);
    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
                                    gconf_client_get_bool (gconf,
                                                           RANDOM_ORDER,
                                                           NULL));
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "toggled", G_CALLBACK(applet_random), qc);

    gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                           gtk_separator_menu_item_new());


#if 0
    item = gtk_image_menu_item_new_with_mnemonic (_("_Edit playlist..."));
    image = gtk_image_new_from_stock (GTK_STOCK_JUSTIFY_FILL,
                                      GTK_ICON_SIZE_MENU);
    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "activate", G_CALLBACK(applet_edit_playlist), qc);

    gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                           gtk_separator_menu_item_new());
#endif


/*
    item = gtk_image_menu_item_new_from_stock(GTK_STOCK_HELP, NULL);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "activate", G_CALLBACK(applet_help), qc);

    item = gtk_menu_item_new_with_mnemonic (_("About..."));
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "activate", G_CALLBACK(applet_about), qc);

    gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                           gtk_separator_menu_item_new());
*/

    item = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    g_signal_connect(item, "activate", G_CALLBACK(menu_quit), NULL);

    gtk_widget_show_all (GTK_WIDGET (menu));

    trayicon = trayicon_new ();

    g_signal_connect(trayicon, "button-press-event",
                     G_CALLBACK(applet_user_event), qc);
    g_signal_connect(trayicon, "scroll-event",
                     G_CALLBACK(applet_user_event), qc);

    g_signal_connect (qc, "playlist-position-changed",
                      G_CALLBACK (applet_song_change), trayicon);

    trayicon_set_name (TRAYICON (trayicon), "strange-quark");
    trayicon_set_menu (TRAYICON (trayicon), menu);
    trayicon_set_image (TRAYICON (trayicon), QUARK_ICON);
    /* XXX set this with the song */
    trayicon_set_tooltip (TRAYICON (trayicon), "Quark");

    trayicon_show (TRAYICON (trayicon));

    if (gconf_client_get_bool (gconf, PLAY_ON_RUN, NULL))
        quarkclient_play (qc, NULL);
}

void
applet_shutdown ()
{
    g_object_unref (G_OBJECT (qc));
    gtk_object_destroy (GTK_OBJECT (trayicon));
}
